#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

typedef struct HidReport
{
    /*
     * +----\------+-----+-----+-----+-----+-----+-----+-----+-----+
     * |Byte \ Bit |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |
     * +------\----+-----+-----+-----+-----+-----+-----+-----+-----+
     * |  0        |                 Light (L) XAxis               |
     * +------\----+-----+-----+-----+-----+-----+-----+-----+-----+
     * |  0        |                Lighty (L) YAxis               |
     * +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
     * |  1        |        UNUSED   | Btn | Light (H) | Lighty (H)|
     * +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
     *
     */
    uint16_t light:10;
    uint16_t lighty:10;
    uint8_t  button:1;
    uint8_t  unused:3;
} HidReport_t;

static HidReport_t reportBuffer;
static uchar idleRate; // Unused

/////////////////////////////////////////////////////////////////////
// USB report descriptor, generated by HID Descriptor Tool from
//    http://www.usb.org/developers/hidpage/dt2_4.zip
/////////////////////////////////////////////////////////////////////
PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = {
    0x05, 0x01,           // USAGE_PAGE (Generic Desktop)
    0x09, 0x04,           // USAGE (Joystick)
    0xa1, 0x01,           // COLLECTION (Application)
    0x05, 0x01,           //   USAGE_PAGE (Generic Desktop)
    0x09, 0x01,           //   USAGE (Pointer)
    0xa1, 0x00,           //   COLLECTION (Physical)
    0x05, 0x01,           //     USAGE_PAGE (Generic Desktop)
    0x09, 0x31,           //     USAGE (X)
    0x09, 0x30,           //     USAGE (Y)
    0x15, 0x00,	          //     LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x03,     //     LOGICAL_MAXIMUM (1023)
    0x75, 0x0a,           //     REPORT_SIZE (10)
    0x95, 0x02,           //     REPORT_COUNT (2)
    0x81, 0x02,           //     INPUT (Data,Var,Abs)
    0x05, 0x09,           //     USAGE_PAGE (Button)
    0x09, 0x01,           //     USAGE (Button 1)
    0x15, 0x00,           //     LOGICAL_MINIMUM (0)
    0x25, 0x01,           //     LOGICAL_MAXIMUM (1)
    0x75, 0x01,           //     REPORT_SIZE (1)
    0x95, 0x01,           //     REPORT_COUNT (1)
    0x81, 0x02,           //     INPUT (Data,Var,Abs)
    0x75, 0x01,           //     REPORT_SIZE (1)
    0x95, 0x03,           //     REPORT_COUNT (3)
    0x81, 0x03,           //     INPUT (Cnst,Var,Abs)
    0xc0,                 //   END_COLLECTION
    0xc0                  // END_COLLECTION
};

///////////////////////////////////////////////////////////
uint16_t read_adc(uint8_t channel)
{
    uint16_t adc_val;

    ADMUX = 0x40;
    ADMUX |= channel;
    ADCSRA = 0xC6;
    while ((ADCSRA & (1<<ADSC)))
        ;
    adc_val = ADCL + ADCH*256;
    return adc_val;
}

///////////////////////////////////////////////////////////
void buildReport()
{
    //reportBuffer.button = ((PINC & (1<<PC3)) == 0);
    //Game Controller
    //Incline Left positive number , Right negative number
    //int32_t Xaxis = ((read_adc(PC0)/2)*(read_adc(PC2)/2)/512)-((read_adc(PC1)/2)*(read_adc(PC3)/2)/512);
    //Incline Front positive number , Back negative number
    //int32_t Yaxis = ((read_adc(PC0)/2)*(read_adc(PC1)/2)/512)-((read_adc(PC2)/2)*(read_adc(PC3)/2)/512);
    //reportBuffer.light = (int16_t)Xaxis;
    ///reportBuffer.lighty = (int16_t)Yaxis;
	int16_t xaxis = ((read_adc(PC0)+read_adc(PC2))/2)-((read_adc(PC1)+read_adc(PC3))/2)+(1023/2);
	int16_t yaxis = ((read_adc(PC0)+read_adc(PC1))/2)-((read_adc(PC2)+read_adc(PC3))/2)+(1023/2);
	reportBuffer.light = yaxis;
	reportBuffer.lighty = xaxis;
}

///////////////////////////////////////////////////////////
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    if(rq->bRequest == USBRQ_HID_GET_REPORT)
    {
        usbMsgPtr = (void *)&reportBuffer;
        return sizeof(reportBuffer);
    }
    else if (rq->bRequest == USBRQ_HID_GET_IDLE)
    {
        usbMsgPtr = &idleRate;
        return 1;
    }
    else if (rq->bRequest == USBRQ_HID_SET_IDLE)
    {
        idleRate = rq->wValue.bytes[1];
    }

    return 0;
}

///////////////////////////////////////////////////////////
int main(void)
{
    DDRC = 0b00000000; // PC0..PC3 Input
   // DDRC  = (1<<PC0)|(1<<PC1)|(1<<PC2); // PC0..PC2 = Output, PC3..PC5 = Input
   // PORTC = (1 << PC3);                 // Enable pull-up at PC3

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        /* always have report ready */
        buildReport();

        usbPoll();

        /* let the host know without asking */
        if (usbInterruptIsReady())
        {
            usbSetInterrupt((void *)&reportBuffer, sizeof(reportBuffer));
        }
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
